﻿#include "LaTeXMathRenderer.h"
#include "Aspose.TeX.Cpp/Features/MathRenderer.h"
#include "Aspose.TeX.Cpp/Features/PngMathRendererOptions.h"


#include <system/string.h>
#include <system/io/text_writer.h>
#include <system/io/path.h>
#include <system/io/memory_stream.h>
#include <system/io/file_stream.h>
#include <system/io/file_mode.h>
#include <system/io/file.h>
#include <system/details/dispose_guard.h>
#include <system/console.h>
#include <drawing/size_f.h>
#include <drawing/color.h>
#include <cstdint>

#include "RunExamples.h"

using namespace Aspose::TeX::Features;

namespace System
{
	namespace IO
	{
		class Stream;
	} // namespace IO
} // namespace System

namespace Aspose {

namespace TeX {

	class MathRenderer;

namespace Examples {

namespace CPP {

namespace LaTeXMathRendering {


RTTI_INFO_IMPL_HASH(1921358231u, ::Aspose::TeX::Examples::CPP::LaTeXMathRendering::LaTeXMathRenderer, ThisTypeBaseTypesInfo);

// Using statement is translated using System::Details::DisposeGuard class which may store exception and then throw from destructor.
// We block the warnings related as these are false alarms (the exception, if caught, will be re-thrown from the destructor).
#if defined(__MSVC__)
#pragma warning( push )
#pragma warning(disable : 4715)
#pragma warning(disable : 4700)
#pragma warning(disable : 4701)
#elif defined(__GNUC__)
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wreturn-type"
#endif
void LaTeXMathRenderer::Run()
{
    // ExStart:LaTeXMathRenderer
    // Create rendering options specifying the image resolution 150 dpi
    System::SharedPtr<MathRendererOptions> options = System::MakeObject<PngMathRendererOptions>();
    // Specify the preamble.
    options->set_Preamble(u"\\usepackage{amsmath}\r\n\\usepackage{amsfonts}\r\n\\usepackage{amssymb}\r\n\\usepackage{color}");
    // Specify the scaling factor 300%.
    options->set_Scale(3000);
    // Specify the foreground color.
    options->set_TextColor(System::Drawing::Color::get_Black());
    // Specify the background color.
    options->set_BackgroundColor(System::Drawing::Color::get_White());
    // Specify the output stream for the log file.
    options->set_LogStream(System::MakeObject<System::IO::MemoryStream>());
    // Specify whether to show the terminal output on the console or not.
    options->set_ShowTerminal(true);
    
    // The variable in which the dimensions of the resulting image will be written.
    System::Drawing::SizeF size;
    // Create the output stream for the formula image.
    {
        System::SharedPtr<System::IO::Stream> stream = System::IO::File::Open(System::IO::Path::Combine(RunExamples::OutputDirectory, u"math-formula.png"), System::IO::FileMode::Create);
        // Clearing resources under 'using' statement
        System::Details::DisposeGuard<1> __dispose_guard_0({ stream});
        // ------------------------------------------
        
        try
        {
			Aspose:TeX::Features::MathRenderer::Render(u"\\begin{equation*}\r\ne^x = x^{\\color{red}0} + x^{\\color{red}1} + \\frac{x^{\\color{red}2}}{2} + \\frac{x^{\\color{red}3}}{6} + \\cdots = \\sum_{n\\geq 0} \\frac{x^{\\color{red}n}}{n!}\r\n\\end{equation*}", stream, options, size);
        }
        catch(...)
        {
            __dispose_guard_0.SetCurrentException(std::current_exception());
        }
    }
    
    // Show other results.
    System::Console::get_Out()->WriteLine(options->get_ErrorReport());
    System::Console::get_Out()->WriteLine();
    System::Console::get_Out()->WriteLine(System::String(u"Size: ") + size);
    // ExEnd:LaTeXMathRenderer
}
#if defined(__MSVC__)
#pragma warning( pop )
#elif defined(__GNUC__)
#pragma GCC diagnostic pop
#endif

} // namespace LaTeXMathRendering
} // namespace CSharp
} // namespace Examples
} // namespace TeX
} // namespace Aspose
