﻿#pragma once
// Copyright (c) 2001-2021 Aspose Pty Ltd. All Rights Reserved.

#include <system/string.h>
#include <drawing/color.h>
#include <cstdint>

#include "Aspose.TeX.Cpp/aspose_tex_api_defs.h"

namespace Aspose
{
namespace TeX
{
namespace Features
{
class MathRenderer;
} // namespace Features
namespace IO
{
class IInputTerminal;
class IOutputWorkingDirectory;
} // namespace IO
namespace Logging
{
class TeXLog;
} // namespace Logging
} // namespace TeX
} // namespace Aspose
namespace System
{
namespace IO
{
class Stream;
} // namespace IO
} // namespace System

namespace Aspose {

namespace TeX {

namespace Features {

/// <summary>
/// Math formula common rendering options.
/// </summary>
class ASPOSE_TEX_SHARED_CLASS MathRendererOptions : public System::Object
{
    typedef MathRendererOptions ThisType;
    typedef System::Object BaseType;
    
    typedef ::System::BaseTypesInfo<BaseType> ThisTypeBaseTypesInfo;
    ASPOSE_TEX_SHARED_RTTI_INFO_DECL();
    
    friend class Aspose::TeX::Features::MathRenderer;
    
public:

    /// <summary>
    /// Gets/sets LaTeX document preamble.
    /// </summary>
    ASPOSE_TEX_SHARED_API System::String get_Preamble() const;
    /// <summary>
    /// Gets/sets LaTeX document preamble.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_Preamble(System::String value);
    /// <summary>
    /// Gets/set the scale. 1000 means 100%, 1200 means 120%, etc.
    /// </summary>
    ASPOSE_TEX_SHARED_API int32_t get_Scale() const;
    /// <summary>
    /// Gets/set the scale. 1000 means 100%, 1200 means 120%, etc.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_Scale(int32_t value);
    /// <summary>
    /// Gets/sets the formula text color.
    /// </summary>
    ASPOSE_TEX_SHARED_API System::Drawing::Color get_TextColor() const;
    /// <summary>
    /// Gets/sets the formula text color.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_TextColor(System::Drawing::Color value);
    /// <summary>
    /// Gets/sets the background color.
    /// </summary>
    ASPOSE_TEX_SHARED_API System::Drawing::Color get_BackgroundColor() const;
    /// <summary>
    /// Gets/sets the background color.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_BackgroundColor(System::Drawing::Color value);
    /// <summary>
    /// Gets/set the stream to write log output to.
    /// </summary>
    ASPOSE_TEX_SHARED_API System::SharedPtr<System::IO::Stream> get_LogStream() const;
    /// <summary>
    /// Gets/set the stream to write log output to.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_LogStream(System::SharedPtr<System::IO::Stream> value);
    /// <summary>
    /// The flag that controls terminal output. If <code>true</code> then terminal output is written to console.
    /// </summary>
    ASPOSE_TEX_SHARED_API bool get_ShowTerminal() const;
    /// <summary>
    /// The flag that controls terminal output. If <code>true</code> then terminal output is written to console.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_ShowTerminal(bool value);
    /// <summary>
    /// Gets the error report.
    /// </summary>
    ASPOSE_TEX_SHARED_API System::String get_ErrorReport() const;
    
    /// <summary>
    /// Creates a new instance.
    /// </summary>
    ASPOSE_TEX_SHARED_API MathRendererOptions();
    
protected:

    /// <summary>
    /// The default LaTeX document preamble.
    /// </summary>
    static System::String DefaultPreamble;
    
    /// <summary>
    /// Gets the error report.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_ErrorReport(System::String value);
    System::SharedPtr<Aspose::TeX::Logging::TeXLog> get_Log() const;
    void set_Log(System::SharedPtr<Aspose::TeX::Logging::TeXLog> value);
    System::SharedPtr<Aspose::TeX::IO::IInputTerminal> get_TermIn() const;
    void set_TermIn(System::SharedPtr<Aspose::TeX::IO::IInputTerminal> value);
    System::SharedPtr<Aspose::TeX::IO::IOutputWorkingDirectory> get_OutputDirectory() const;
    void set_OutputDirectory(System::SharedPtr<Aspose::TeX::IO::IOutputWorkingDirectory> value);
    float get_Margin() const;
    void set_Margin(float value);
    
    #ifdef ASPOSE_GET_SHARED_MEMBERS
    ASPOSE_TEX_SHARED_API System::Object::shared_members_type GetSharedMembers() override;
    #endif
    
    
private:

    System::String _preamble;
    int32_t _scale;
    System::Drawing::Color _textColor;
    System::Drawing::Color _backgroundColor;
    System::SharedPtr<System::IO::Stream> _logStream;
    bool _showTerminal;
    System::String _errorReport;
    System::SharedPtr<Aspose::TeX::Logging::TeXLog> _log;
    System::SharedPtr<Aspose::TeX::IO::IInputTerminal> _termIn;
    System::SharedPtr<Aspose::TeX::IO::IOutputWorkingDirectory> _outputDirectory;
    float _margin;
    
};

} // namespace Features
} // namespace TeX
} // namespace Aspose


