﻿#pragma once
// Copyright (c) 2001-2021 Aspose Pty Ltd. All Rights Reserved.

#include <system/string.h>

#include "Aspose.TeX.Cpp/IO/IInputWorkingDirectory.h"
#include "Aspose.TeX.Cpp/aspose_tex_api_defs.h"

namespace Aspose
{
namespace Pal
{
class PalZipReader;
} // namespace Pal
namespace TeX
{
namespace IO
{
class ZipNode;
class ZipTree;
} // namespace IO
} // namespace TeX
} // namespace Aspose
namespace System
{
namespace IO
{
class Stream;
} // namespace IO
} // namespace System

namespace Aspose {

namespace TeX {

namespace IO {

/// <summary>
/// Implements the method for getting a file stream to write to when working directory is a ZIP archive.
/// </summary>
class ASPOSE_TEX_SHARED_CLASS InputZipDirectory : public Aspose::TeX::IO::IInputWorkingDirectory
{
    typedef InputZipDirectory ThisType;
    typedef Aspose::TeX::IO::IInputWorkingDirectory BaseType;
    
    typedef ::System::BaseTypesInfo<BaseType> ThisTypeBaseTypesInfo;
    ASPOSE_TEX_SHARED_RTTI_INFO_DECL();
    
    friend class Aspose::TeX::IO::ZipNode;
    friend class Aspose::TeX::IO::ZipTree;
    
public:

    /// <summary>
    /// Creates new instance.
    /// </summary>
    /// <param name="zipStream">The stream to write the archive to.</param>
    /// <param name="basePath">The base path inside the ZIP archive.</param>
    ASPOSE_TEX_SHARED_API InputZipDirectory(System::SharedPtr<System::IO::Stream> zipStream, System::String basePath);
    
    /// <summary>
    /// Returns the stream to read from.
    /// </summary>
    /// <param name="fileName">The file name.</param>
    /// <param name="fullName">The full file name.</param>
    /// <param name="searchSubdirectories">Indicates whether to look for a file in subdirectories.</param>
    /// <returns>The stream.</returns>
    ASPOSE_TEX_SHARED_API System::SharedPtr<System::IO::Stream> GetFile(System::String fileName, System::String& fullName, bool searchSubdirectories = false) override;
    /// <summary>
    /// Disposes the instance.
    /// </summary>
    ASPOSE_TEX_SHARED_API void Dispose() override;
    
protected:

    static char16_t SeparatorChar;
    
    #ifdef ASPOSE_GET_SHARED_MEMBERS
    ASPOSE_TEX_SHARED_API System::Object::shared_members_type GetSharedMembers() override;
    #endif
    
    
private:

    System::SharedPtr<System::IO::Stream> _zipStream;
    System::SharedPtr<Aspose::Pal::PalZipReader> _zipReader;
    System::String _basePath;
    System::SharedPtr<ZipTree> _tree;
    
    void LoadResources();
    
};

} // namespace IO
} // namespace TeX
} // namespace Aspose


