﻿#pragma once
// Copyright (c) 2001-2021 Aspose Pty Ltd. All Rights Reserved.

#include <system/array.h>
#include <cstdint>

#include "Aspose.TeX.Cpp/aspose_tex_api_defs.h"

namespace Aspose
{
namespace Foundation
{
namespace Drawing
{
class DrFont;
} // namespace Drawing
} // namespace Foundation
namespace TeX
{
namespace IO
{
class IOutputWorkingDirectory;
} // namespace IO
namespace Presentation
{
namespace Aps
{
class ApsDevice;
} // namespace Aps
class DviAdapter;
class GlyphData;
namespace Image
{
class ImageDevice;
} // namespace Image
namespace Pdf
{
class PdfDevice;
} // namespace Pdf
class SaveOptions;
class TeXPresenter;
namespace Xps
{
class XpsDevice;
} // namespace Xps
} // namespace Presentation
class TeXEngine;
} // namespace TeX
} // namespace Aspose
namespace System
{
namespace Collections
{
namespace Generic
{
template <typename> class List;
} // namespace Generic
} // namespace Collections
namespace Drawing
{
class Brush;
namespace Drawing2D
{
class GraphicsPath;
class Matrix;
} // namespace Drawing2D
class Pen;
class PointF;
class RectangleF;
class SizeF;
} // namespace Drawing
} // namespace System

namespace Aspose {

namespace TeX {

namespace Presentation {

/// <summary>
/// The <b>Aspose.TeX.Presentation</b> namespace provides common classes for typesetting TeX files to other formats.
/// </summary>
class NamespaceDoc : public System::Object
{
    typedef NamespaceDoc ThisType;
    typedef System::Object BaseType;
    
    typedef ::System::BaseTypesInfo<BaseType> ThisTypeBaseTypesInfo;
    RTTI_INFO_DECL();
    
};

/// <summary>
/// Implements the interface for outputting text and graphic content
/// to abstract device. Rendering is performed page by page.
/// </summary>
class ASPOSE_TEX_SHARED_CLASS Device : public virtual System::Object
{
    typedef Device ThisType;
    typedef System::Object BaseType;
    
    typedef ::System::BaseTypesInfo<BaseType> ThisTypeBaseTypesInfo;
    ASPOSE_TEX_SHARED_RTTI_INFO_DECL();
    
    friend class Aspose::TeX::TeXEngine;
    friend class Aspose::TeX::Presentation::Aps::ApsDevice;
    friend class Aspose::TeX::Presentation::DviAdapter;
    friend class Aspose::TeX::Presentation::Image::ImageDevice;
    friend class Aspose::TeX::Presentation::Pdf::PdfDevice;
    friend class Aspose::TeX::Presentation::TeXPresenter;
    friend class Aspose::TeX::Presentation::Xps::XpsDevice;
    
public:

    /// <summary>
    /// Gets the number of pages.
    /// </summary>
    virtual ASPOSE_TEX_SHARED_API int32_t get_PageCount() = 0;
    /// <summary>
    /// Shows if device is ready for output.
    /// </summary>
    virtual ASPOSE_TEX_SHARED_API bool get_IsReady() = 0;
    /// <summary>
    /// Gets destination name: output file name or device description.
    /// </summary>
    virtual ASPOSE_TEX_SHARED_API System::String get_DestinationName() = 0;
    /// <summary>
    /// Gets/sets the current stroke.
    /// </summary>
    virtual ASPOSE_TEX_SHARED_API System::SharedPtr<System::Drawing::Pen> get_Stroke();
    /// <summary>
    /// Gets/sets the current stroke.
    /// </summary>
    virtual ASPOSE_TEX_SHARED_API void set_Stroke(System::SharedPtr<System::Drawing::Pen> value);
    /// <summary>
    /// Gets/sets the current fill.
    /// </summary>
    virtual ASPOSE_TEX_SHARED_API System::SharedPtr<System::Drawing::Brush> get_Fill();
    /// <summary>
    /// Gets/sets the current fill.
    /// </summary>
    virtual ASPOSE_TEX_SHARED_API void set_Fill(System::SharedPtr<System::Drawing::Brush> value);
    
    /// <summary>
    /// Creates a new instance.
    /// </summary>
    ASPOSE_TEX_SHARED_API Device();
    
    /// <summary>
    /// Initializes device.
    /// </summary>
    virtual ASPOSE_TEX_SHARED_API void Init() = 0;
    /// <summary>
    /// Creates a copy of this device.
    /// </summary>
    /// <returns>Copy of this device.</returns>
    virtual ASPOSE_TEX_SHARED_API System::SharedPtr<Device> Create();
    /// <summary>
    /// Disposes the device.
    /// </summary>
    virtual ASPOSE_TEX_SHARED_API void Dispose();
    /// <summary>
    /// Starts the whole document.
    /// </summary>
    virtual ASPOSE_TEX_SHARED_API void StartDocument() = 0;
    /// <summary>
    /// Finalizes the whole document.
    /// </summary>
    virtual ASPOSE_TEX_SHARED_API void EndDocument() = 0;
    /// <summary>
    /// Starts a new page.
    /// </summary>
    /// <param name="width">The page width.</param>
    /// <param name="height">The page height.</param>
    virtual ASPOSE_TEX_SHARED_API void StartPage(float width, float height) = 0;
    /// <summary>
    /// Finalizes a page.
    /// </summary>
    virtual ASPOSE_TEX_SHARED_API void EndPage() = 0;
    /// <summary>
    /// Sets the hyperlink with a URI as its target.
    /// </summary>
    /// <param name="activeRect">The active rectangle of the link.</param>
    /// <param name="border">The link border.</param>
    /// <param name="targetUri">The target URI.</param>
    virtual ASPOSE_TEX_SHARED_API void AddHyperlink(System::Drawing::RectangleF activeRect, System::SharedPtr<System::Drawing::Pen> border, System::String targetUri) = 0;
    /// <summary>
    /// Adds the bookmark identified by the name.
    /// </summary>
    /// <param name="name">The name.</param>
    /// <param name="position">The position.</param>
    virtual ASPOSE_TEX_SHARED_API void AddBookmark(System::String name, System::Drawing::PointF position) = 0;
    /// <summary>
    /// Sets the current coordinate space transformation.
    /// </summary>
    /// <param name="matrix">A transformation matrix.</param>
    virtual ASPOSE_TEX_SHARED_API void SetTransform(System::SharedPtr<System::Drawing::Drawing2D::Matrix> matrix) = 0;
    /// <summary>
    /// Sets the current clip path.
    /// </summary>
    /// <param name="path">A clip path.</param>
    virtual ASPOSE_TEX_SHARED_API void SetClip(System::SharedPtr<System::Drawing::Drawing2D::GraphicsPath> path) = 0;
    /// <summary>
    /// Draws a text string.
    /// </summary>
    /// <param name="str">The string.</param>
    /// <param name="originX">The x coordinate of the origin.</param>
    /// <param name="originY">The x coordinate of the origin.</param>
    /// <param name="charInfos">Glyph data required for precise typesetting of a text string.</param>
    virtual ASPOSE_TEX_SHARED_API void DrawString(System::String str, float originX, float originY, System::SharedPtr<System::Collections::Generic::List<System::SharedPtr<GlyphData>>> charInfos) = 0;
    /// <summary>
    /// Draws a path.
    /// </summary>
    /// <param name="path">A path to draw.</param>
    virtual ASPOSE_TEX_SHARED_API void DrawPath(System::SharedPtr<System::Drawing::Drawing2D::GraphicsPath> path) = 0;
    /// <summary>
    /// Fills a path.
    /// </summary>
    /// <param name="path">A path to fill.</param>
    virtual ASPOSE_TEX_SHARED_API void FillPath(System::SharedPtr<System::Drawing::Drawing2D::GraphicsPath> path) = 0;
    /// <summary>
    /// Shows a raster image.
    /// </summary>
    /// <param name="origin">The bottom-left corner of the shown image.</param>
    /// <param name="size">The size of the shown image.</param>
    /// <param name="imageData">The image data.</param>
    virtual ASPOSE_TEX_SHARED_API void ShowImage(System::Drawing::PointF origin, System::Drawing::SizeF size, System::ArrayPtr<uint8_t> imageData) = 0;
    
protected:

    System::SharedPtr<Aspose::TeX::Presentation::SaveOptions> get_SaveOptions() const;
    void set_SaveOptions(System::SharedPtr<Aspose::TeX::Presentation::SaveOptions> value);
    System::String get_JobName() const;
    void set_JobName(System::String value);
    System::SharedPtr<Aspose::TeX::IO::IOutputWorkingDirectory> get_OutputDirectory() const;
    void set_OutputDirectory(System::SharedPtr<Aspose::TeX::IO::IOutputWorkingDirectory> value);
    /// <summary>
    /// Gets/sets the current font.
    /// </summary>
    System::SharedPtr<Aspose::Foundation::Drawing::DrFont> get_Font();
    /// <summary>
    /// Gets/sets the current font.
    /// </summary>
    void set_Font(System::SharedPtr<Aspose::Foundation::Drawing::DrFont> value);
    
    /// <summary>
    /// Clone constructor. Initializes this device with another device.
    /// </summary>
    /// <param name="device">The device to clone.</param>
    ASPOSE_TEX_SHARED_API Device(System::SharedPtr<Device> device);
    
    virtual ASPOSE_TEX_SHARED_API void EndPageInternal(System::String evalMark);
    /// <summary>
    /// Saves the current graphics state.
    /// </summary>
    virtual ASPOSE_TEX_SHARED_API void SaveGraphicsState();
    /// <summary>
    /// Rolls back to the graphics state previously saved.
    /// </summary>
    virtual ASPOSE_TEX_SHARED_API void RestoreGraphicsState();
    #ifdef ASPOSE_GET_SHARED_MEMBERS
    ASPOSE_TEX_SHARED_API System::Object::shared_members_type GetSharedMembers() override;
    #endif
    
    
private:

    System::SharedPtr<Aspose::TeX::Presentation::SaveOptions> _saveOptions;
    System::String _jobName;
    System::SharedPtr<Aspose::TeX::IO::IOutputWorkingDirectory> _outputDirectory;
    System::ArrayPtr<System::SharedPtr<Aspose::Foundation::Drawing::DrFont>> _font;
    System::SharedPtr<System::Drawing::Pen> _stroke;
    System::SharedPtr<System::Drawing::Brush> _fill;
    
};

} // namespace Presentation
} // namespace TeX
} // namespace Aspose


