﻿#pragma once
// Copyright (c) 2001-2021 Aspose Pty Ltd. All Rights Reserved.

#include "Aspose.TeX.Cpp/Presentation/Device.h"
#include "Aspose.TeX.Cpp/aspose_tex_api_defs.h"

namespace Aspose
{
namespace TeX
{
namespace Presentation
{
namespace Aps
{
class ApsDevice;
} // namespace Aps
class GlyphData;
} // namespace Presentation
} // namespace TeX
} // namespace Aspose
namespace System
{
namespace Collections
{
namespace Generic
{
template <typename> class List;
} // namespace Generic
} // namespace Collections
namespace Drawing
{
class Brush;
namespace Drawing2D
{
class GraphicsPath;
class Matrix;
} // namespace Drawing2D
class Pen;
class PointF;
class RectangleF;
class SizeF;
} // namespace Drawing
namespace IO
{
class Stream;
} // namespace IO
} // namespace System

namespace Aspose {

namespace TeX {

namespace Presentation {

namespace Xps {

/// <summary>
/// The <b>Aspose.TeX.Presentation.Xps</b> namespace provides classes for typesetting TeX files to XPS.
/// </summary>
class NamespaceDoc : public System::Object
{
    typedef NamespaceDoc ThisType;
    typedef System::Object BaseType;
    
    typedef ::System::BaseTypesInfo<BaseType> ThisTypeBaseTypesInfo;
    RTTI_INFO_DECL();
    
};

/// <summary>
/// Implements the interface for outputting text and graphic content to XPS document.
/// </summary>
class ASPOSE_TEX_SHARED_CLASS XpsDevice : public Aspose::TeX::Presentation::Device
{
    typedef XpsDevice ThisType;
    typedef Aspose::TeX::Presentation::Device BaseType;
    
    typedef ::System::BaseTypesInfo<BaseType> ThisTypeBaseTypesInfo;
    ASPOSE_TEX_SHARED_RTTI_INFO_DECL();
    
public:

    /// <summary>
    /// Gets the number of pages.
    /// </summary>
    ASPOSE_TEX_SHARED_API int32_t get_PageCount() override;
    /// <summary>
    /// Shows if device is ready for output.
    /// </summary>
    ASPOSE_TEX_SHARED_API bool get_IsReady() override;
    /// <summary>
    /// Gets destination name: output file name or device description.
    /// </summary>
    ASPOSE_TEX_SHARED_API System::String get_DestinationName() override;
    /// <summary>
    /// Gets/sets the current stroke.
    /// </summary>
    ASPOSE_TEX_SHARED_API System::SharedPtr<System::Drawing::Pen> get_Stroke() override;
    /// <summary>
    /// Gets/sets the current stroke.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_Stroke(System::SharedPtr<System::Drawing::Pen> value) override;
    /// <summary>
    /// Gets/sets the current fill.
    /// </summary>
    ASPOSE_TEX_SHARED_API System::SharedPtr<System::Drawing::Brush> get_Fill() override;
    /// <summary>
    /// Gets/sets the current fill.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_Fill(System::SharedPtr<System::Drawing::Brush> value) override;
    
    /// <summary>
    /// Creates new instance.
    /// The output file will be written to the output working
    /// directory taking the job name as a file name.
    /// </summary>
    ASPOSE_TEX_SHARED_API XpsDevice();
    /// <summary>
    /// Creates new instance.
    /// The output file will be written to specified stream.
    /// </summary>
    /// <param name="stream">The stream to write the output file to.</param>
    ASPOSE_TEX_SHARED_API XpsDevice(System::SharedPtr<System::IO::Stream> stream);
    
    /// <summary>
    /// Initializes device.
    /// </summary>
    ASPOSE_TEX_SHARED_API void Init() override;
    /// <summary>
    /// Creates a copy of this device.
    /// </summary>
    /// <returns>Copy of this device.</returns>
    ASPOSE_TEX_SHARED_API System::SharedPtr<Device> Create() override;
    /// <summary>
    /// Disposes this device instance. Finalizes this device instance graphics state,
    /// i.e. switches APS composing context to the <see cref="ApsCanvas"></see> of the level higher then this
    /// device's graphics state <see cref="ApsCanvas"></see>.
    /// </summary>
    ASPOSE_TEX_SHARED_API void Dispose() override;
    /// <summary>
    /// Starts the whole document.
    /// </summary>
    ASPOSE_TEX_SHARED_API void StartDocument() override;
    /// <summary>
    /// Finalizes the whole document.
    /// </summary>
    ASPOSE_TEX_SHARED_API void EndDocument() override;
    /// <summary>
    /// Starts a new page.
    /// </summary>
    /// <param name="width">The page width.</param>
    /// <param name="height">The page height.</param>
    ASPOSE_TEX_SHARED_API void StartPage(float width, float height) override;
    /// <summary>
    /// Finalizes a page.
    /// </summary>
    ASPOSE_TEX_SHARED_API void EndPage() override;
    /// <summary>
    /// Set the hyperlink with a URI as its target.
    /// </summary>
    /// <param name="activeRect">The active rectangle of the link.</param>
    /// <param name="border">The link border.</param>
    /// <param name="targetUri">The target URI.</param>
    ASPOSE_TEX_SHARED_API void AddHyperlink(System::Drawing::RectangleF activeRect, System::SharedPtr<System::Drawing::Pen> border, System::String targetUri) override;
    /// <summary>
    /// Adds the bookmark identified by the name.
    /// </summary>
    /// <param name="name">The name.</param>
    /// <param name="position">The position.</param>
    ASPOSE_TEX_SHARED_API void AddBookmark(System::String name, System::Drawing::PointF position) override;
    /// <summary>
    /// Sets the current coordinate space transformation.
    /// </summary>
    /// <param name="matrix">A transformation matrix.</param>
    ASPOSE_TEX_SHARED_API void SetTransform(System::SharedPtr<System::Drawing::Drawing2D::Matrix> matrix) override;
    /// <summary>
    /// Sets the current clip path.
    /// </summary>
    /// <param name="path">A clip path.</param>
    ASPOSE_TEX_SHARED_API void SetClip(System::SharedPtr<System::Drawing::Drawing2D::GraphicsPath> path) override;
    /// <summary>
    /// Draws a text string.
    /// </summary>
    /// <param name="str">The string.</param>
    /// <param name="originX">The x coordinate of the origin.</param>
    /// <param name="originY">The x coordinate of the origin.</param>
    /// <param name="charInfos">Glyph data required for precise typesetting of a text string.</param>
    ASPOSE_TEX_SHARED_API void DrawString(System::String str, float originX, float originY, System::SharedPtr<System::Collections::Generic::List<System::SharedPtr<GlyphData>>> charInfos) override;
    /// <summary>
    /// Draws a path.
    /// </summary>
    /// <param name="path">A path to draw.</param>
    ASPOSE_TEX_SHARED_API void DrawPath(System::SharedPtr<System::Drawing::Drawing2D::GraphicsPath> path) override;
    /// <summary>
    /// Fill a path.
    /// </summary>
    /// <param name="path">A path to fill.</param>
    ASPOSE_TEX_SHARED_API void FillPath(System::SharedPtr<System::Drawing::Drawing2D::GraphicsPath> path) override;
    /// <summary>
    /// Shows a raster image.
    /// </summary>
    /// <param name="origin">The bottom-left corner of the shown image.</param>
    /// <param name="size">The size of the shown image.</param>
    /// <param name="imageData">The image data.</param>
    ASPOSE_TEX_SHARED_API void ShowImage(System::Drawing::PointF origin, System::Drawing::SizeF size, System::ArrayPtr<uint8_t> imageData) override;
    
protected:

    /// <summary>
    /// Clone constructor. Initializes this device with another device.
    /// </summary>
    /// <param name="device">The device to clone.</param>
    ASPOSE_TEX_SHARED_API XpsDevice(System::SharedPtr<XpsDevice> device);
    
    ASPOSE_TEX_SHARED_API MEMBER_FUNCTION_MAKE_OBJECT_DECLARATION(XpsDevice, CODEPORTING_ARGS(System::SharedPtr<XpsDevice> device));
    ASPOSE_TEX_SHARED_API void EndPageInternal(System::String evalMark) override;
    /// <summary>
    /// Shows a raster image.
    /// </summary>
    /// <param name="origin">The bottom-left corner of the shown image.</param>
    /// <param name="size">The size of the shown image.</param>
    /// <param name="imageData">The image data.</param>
    /// <param name="cropBox">The crop box.</param>
    void ShowImage(System::Drawing::PointF origin, System::Drawing::SizeF size, System::ArrayPtr<uint8_t> imageData, System::Drawing::RectangleF cropBox);
    
    virtual ASPOSE_TEX_SHARED_API ~XpsDevice();
    
    #ifdef ASPOSE_GET_SHARED_MEMBERS
    ASPOSE_TEX_SHARED_API System::Object::shared_members_type GetSharedMembers() override;
    #endif
    
    
private:

    System::SharedPtr<Aspose::TeX::Presentation::Aps::ApsDevice> _apsDevice;
    System::SharedPtr<System::IO::Stream> _stream;
    System::String _destinationName;
    
    void SaveApsAsXps();
    
};

} // namespace Xps
} // namespace Presentation
} // namespace TeX
} // namespace Aspose


