﻿#pragma once
// Copyright (c) 2001-2021 Aspose Pty Ltd. All Rights Reserved.

#include <system/string.h>
#include <system/io/text_reader.h>
#include <system/idisposable.h>

#include "Aspose.TeX.Cpp/aspose_tex_api_defs.h"

namespace Aspose
{
namespace TeX
{
namespace IO
{
class IInputWorkingDirectory;
class InputZipDirectoryTests;
class IO;
} // namespace IO
namespace Reading
{
class TeXReader;
} // namespace Reading
class TeXEngine;
class TeXExtension;
} // namespace TeX
} // namespace Aspose
namespace System
{
namespace IO
{
class Stream;
} // namespace IO
} // namespace System

namespace Aspose {

namespace TeX {

namespace ResourceProviders {

/// <summary>
/// The <b>Aspose.TeX.ResourceProviders</b> namespace provides common classes for resource providers.
/// </summary>
class NamespaceDoc : public System::Object
{
    typedef NamespaceDoc ThisType;
    typedef System::Object BaseType;
    
    typedef ::System::BaseTypesInfo<BaseType> ThisTypeBaseTypesInfo;
    RTTI_INFO_DECL();
    
};

/// <summary>
/// Class providing TeX format.
/// </summary>
class ASPOSE_TEX_SHARED_CLASS FormatProvider : public System::IDisposable
{
    typedef FormatProvider ThisType;
    typedef System::IDisposable BaseType;
    
    typedef ::System::BaseTypesInfo<BaseType> ThisTypeBaseTypesInfo;
    ASPOSE_TEX_SHARED_RTTI_INFO_DECL();
    
    friend class Aspose::TeX::TeXEngine;
    friend class Aspose::TeX::Reading::TeXReader;
    friend class Aspose::TeX::TeXExtension;
    friend class Aspose::TeX::IO::InputZipDirectoryTests;
    
protected:

    class RelaxReader : public System::IO::TextReader
    {
        typedef RelaxReader ThisType;
        typedef System::IO::TextReader BaseType;
        
        typedef ::System::BaseTypesInfo<BaseType> ThisTypeBaseTypesInfo;
        RTTI_INFO_DECL();
        
    public:
    
        RelaxReader();
        
        System::String ReadLine() override;
        
    private:
    
        bool _eof;
        
    };
    
    
public:

    /// <summary>
    /// Default format provider for ObjectTeX engine extension.
    /// </summary>
    static ASPOSE_TEX_SHARED_API System::SharedPtr<FormatProvider> get_ObjectTeX();
    /// <summary>
    /// LaTeX format provider for ObjectTeX engine extension.
    /// </summary>
    static ASPOSE_TEX_SHARED_API System::SharedPtr<FormatProvider> get_ObjectLaTeX();
    
    /// <summary>
    /// Creates a new instance.
    /// </summary>
    /// <param name="workingDirectory">The working directory.</param>
    /// <param name="formatFileName">The source file name to start reading format from.</param>
    ASPOSE_TEX_SHARED_API FormatProvider(System::SharedPtr<Aspose::TeX::IO::IInputWorkingDirectory> workingDirectory, System::String formatFileName);
    
    /// <summary>
    /// Disposes the instance.
    /// </summary>
    ASPOSE_TEX_SHARED_API void Dispose() override;
    
protected:

    static System::SharedPtr<FormatProvider> get_OriginalPlainTeX();
    static System::SharedPtr<FormatProvider> get_ObjectTeXPlainTeX();
    /// <summary>
    /// Gets format file name.
    /// </summary>
    System::String get_FormatFileName() const;
    /// <summary>
    /// Gets the format name.
    /// </summary>
    System::String get_FormatName() const;
    
    /// <summary>
    /// Gets the stream containing format file.
    /// </summary>
    /// <returns>The stream containing format file.</returns>
    virtual ASPOSE_TEX_SHARED_API System::SharedPtr<System::IO::Stream> GetFormat(System::String fileName = nullptr);
    /// <summary>
    /// Gets auxiliary input source files.
    /// </summary>
    /// <param name="fileName"></param>
    /// <returns></returns>
    System::SharedPtr<System::IO::TextReader> GetInput(System::String fileName, System::SharedPtr<IO::IO> io = nullptr);
    /// <summary>
    /// Gets a file by its name.
    /// </summary>
    /// <param name="filePath">The file path.</param>
    /// <param name="searchSubdirectories"></param>
    /// <returns>The file stream.</returns>
    System::SharedPtr<System::IO::Stream> GetFile(System::String filePath, bool searchSubdirectories = true);
    #ifdef ASPOSE_GET_SHARED_MEMBERS
    ASPOSE_TEX_SHARED_API System::Object::shared_members_type GetSharedMembers() override;
    #endif
    
    
private:

    System::SharedPtr<Aspose::TeX::IO::IInputWorkingDirectory> _workingDirectory;
    System::String _formatFileName;
    System::String _formatName;
    System::SharedPtr<System::IO::TextReader> _relaxReader;
    
};

/// <summary>
/// Format provider that makes it possible to load a format from its TeX sources.
/// </summary>
class TeXSourceFormatProvider : public Aspose::TeX::ResourceProviders::FormatProvider
{
    typedef TeXSourceFormatProvider ThisType;
    typedef Aspose::TeX::ResourceProviders::FormatProvider BaseType;
    
    typedef ::System::BaseTypesInfo<BaseType> ThisTypeBaseTypesInfo;
    RTTI_INFO_DECL();
    
public:

    /// <summary>
    /// Creates a new instance.
    /// </summary>
    /// <param name="workingDirectory">The working directory.</param>
    /// <param name="formatFileName">The source file name to start reading format from.</param>
    TeXSourceFormatProvider(System::SharedPtr<Aspose::TeX::IO::IInputWorkingDirectory> workingDirectory, System::String formatFileName);
    
    /// <summary>
    /// Gets the stream containing format file.
    /// </summary>
    /// <returns>The stream containing format file.</returns>
    System::SharedPtr<System::IO::Stream> GetFormat(System::String fileName = nullptr) override;
    
};

} // namespace ResourceProviders
} // namespace TeX
} // namespace Aspose


