﻿#pragma once
// Copyright (c) 2001-2021 Aspose Pty Ltd. All Rights Reserved.

#include <system/object.h>

#include "Aspose.TeX.Cpp/aspose_tex_api_defs.h"

namespace Aspose
{
namespace TeX
{
namespace Features
{
class MathRenderer;
} // namespace Features
namespace Presentation
{
class TeXPresenter;
} // namespace Presentation
namespace Reading
{
namespace EqtbDefinitions
{
class EqtbDefinition;
} // namespace EqtbDefinitions
class ReadingContext;
class TeXReader;
} // namespace Reading
namespace ResourceProviders
{
class FormatProvider;
class ResourceProvider;
} // namespace ResourceProviders
namespace Tests
{
namespace PlainTeX
{
class PlainTeXFormatTests;
} // namespace PlainTeX
namespace Reading
{
class EquivTests;
class ReadingContextTests;
} // namespace Reading
class TestBase;
namespace VirTeX
{
class IOTests;
class VirTeXFormatTests;
class VirTeXTracingsTests;
class VirTeXVisualTests;
} // namespace VirTeX
} // namespace Tests
class TeXEngine;
class TeXExtension;
class TeXJob;
} // namespace TeX
} // namespace Aspose

namespace Aspose {

namespace TeX {

/// <summary>
/// Class providing available TeX configurations.
/// </summary>
class ASPOSE_TEX_SHARED_CLASS TeXConfig : public System::Object
{
    typedef TeXConfig ThisType;
    typedef System::Object BaseType;
    
    typedef ::System::BaseTypesInfo<BaseType> ThisTypeBaseTypesInfo;
    ASPOSE_TEX_SHARED_RTTI_INFO_DECL();
    
    friend class Aspose::TeX::Features::MathRenderer;
    friend class Aspose::TeX::TeXEngine;
    friend class Aspose::TeX::Reading::ReadingContext;
    friend class Aspose::TeX::Reading::TeXReader;
    friend class Aspose::TeX::TeXJob;
    friend class Aspose::TeX::Tests::TestBase;
    friend class Aspose::TeX::Tests::PlainTeX::PlainTeXFormatTests;
    friend class Aspose::TeX::Tests::Reading::EquivTests;
    friend class Aspose::TeX::Tests::Reading::ReadingContextTests;
    friend class Aspose::TeX::Tests::VirTeX::IOTests;
    friend class Aspose::TeX::Tests::VirTeX::VirTeXTracingsTests;
    friend class Aspose::TeX::Tests::VirTeX::VirTeXFormatTests;
    friend class Aspose::TeX::Tests::VirTeX::VirTeXVisualTests;
    
public:

    /// <summary>
    /// Gets the configuration of ObjectTeX engine extension with no format preloaded, i.e. in INITEX mode.
    /// Supposed to be used for format preparation.
    /// </summary>
    static ASPOSE_TEX_SHARED_API System::SharedPtr<TeXConfig> get_ObjectIniTeX();
    /// <summary>
    /// Gets the configuration of ObjectTeX engine extension with ObjectLaTeX format preloaded.
    /// </summary>
    static ASPOSE_TEX_SHARED_API System::SharedPtr<TeXConfig> get_ObjectLaTeX();
    
    /// <summary>
    /// Gets the configuration of ObjectTeX engine extension with provided format preloaded.
    /// </summary>
    /// <param name="formatProvider">The format provider.
    /// If left equal to null then default ObjectTeX format (which is essentially Plain TeX) will be loaded.</param>
    /// <returns>"ObjectTeX engine extension with provided format preloaded" configuration.</returns>
    static ASPOSE_TEX_SHARED_API System::SharedPtr<TeXConfig> ObjectTeX(System::SharedPtr<Aspose::TeX::ResourceProviders::FormatProvider> formatProvider = nullptr);
    
protected:

    System::SharedPtr<Aspose::TeX::TeXExtension> get_TeXExtension() const;
    System::SharedPtr<Aspose::TeX::Reading::EqtbDefinitions::EqtbDefinition> get_EqtbDefinition();
    System::SharedPtr<Aspose::TeX::ResourceProviders::FormatProvider> get_FormatProvider() const;
    /// <summary>
    /// Presenter's prototype.
    /// </summary>
    System::SharedPtr<Presentation::TeXPresenter> get_Presenter() const;
    /// <summary>
    /// Presenter's prototype.
    /// </summary>
    void set_Presenter(System::SharedPtr<Presentation::TeXPresenter> value);
    System::SharedPtr<ResourceProviders::ResourceProvider> get_FontProvider() const;
    /// <summary>
    /// Virgin TeX engine with no format preloaded.
    /// Used in tests only.
    /// </summary>
    /// <returns></returns>
    static System::SharedPtr<TeXConfig> get_VirTeX();
    
    /// <summary>
    /// Provided format under original TeX engine.
    /// If provided format is .fmt and format preloading fails then Plain TeX is preloaded as default format.
    /// </summary>
    /// <param name="provider">The format provider.</param>
    /// <returns>"Provided format under original TeX engine" config.</returns>
    static System::SharedPtr<TeXConfig> OriginalTeX(System::SharedPtr<Aspose::TeX::ResourceProviders::FormatProvider> provider);
    #ifdef ASPOSE_GET_SHARED_MEMBERS
    ASPOSE_TEX_SHARED_API System::Object::shared_members_type GetSharedMembers() override;
    #endif
    
    
private:

    System::SharedPtr<Aspose::TeX::TeXExtension> _texExtension;
    System::SharedPtr<Aspose::TeX::ResourceProviders::FormatProvider> _formatProvider;
    System::SharedPtr<Presentation::TeXPresenter> _presenter;
    System::SharedPtr<ResourceProviders::ResourceProvider> _fontProvider;
    
    TeXConfig(System::SharedPtr<Aspose::TeX::TeXExtension> texExtension, System::SharedPtr<Presentation::TeXPresenter> presenter, System::SharedPtr<Aspose::TeX::ResourceProviders::FormatProvider> formatProvider);
    
    MEMBER_FUNCTION_MAKE_OBJECT_DECLARATION(TeXConfig, CODEPORTING_ARGS(System::SharedPtr<Aspose::TeX::TeXExtension> texExtension, System::SharedPtr<Presentation::TeXPresenter> presenter, System::SharedPtr<Aspose::TeX::ResourceProviders::FormatProvider> formatProvider));
    
};

} // namespace TeX
} // namespace Aspose


