﻿#pragma once
// Copyright (c) 2001-2021 Aspose Pty Ltd. All Rights Reserved.

#include <system/object.h>

#include "Aspose.TeX.Cpp/aspose_tex_api_defs.h"

namespace Aspose
{
namespace TeX
{
namespace Presentation
{
class Device;
} // namespace Presentation
class TeXEngine;
class TeXOptions;
} // namespace TeX
} // namespace Aspose
namespace System
{
namespace IO
{
class Stream;
} // namespace IO
class String;
} // namespace System

namespace Aspose {

namespace TeX {

/// <summary>
/// Implements features of a TeX job.
/// </summary>
class ASPOSE_TEX_SHARED_CLASS TeXJob : public System::Object
{
    typedef TeXJob ThisType;
    typedef System::Object BaseType;
    
    typedef ::System::BaseTypesInfo<BaseType> ThisTypeBaseTypesInfo;
    ASPOSE_TEX_SHARED_RTTI_INFO_DECL();
    
public:

    /// <summary>
    /// Creates a TeX job for running the engine in production mode to typeset a TeX file.
    /// </summary>
    /// <param name="stream">The stream containing the TeX file.</param>
    /// <param name="device">The device defining output representation.</param>
    /// <param name="options">TeX engine run options.</param>
    ASPOSE_TEX_SHARED_API TeXJob(System::SharedPtr<System::IO::Stream> stream, System::SharedPtr<Presentation::Device> device, System::SharedPtr<TeXOptions> options);
    /// <summary>
    /// Creates a TeX job for running the engine in production mode to typeset a TeX file.
    /// </summary>
    /// <param name="path">The path to the TeX file.</param>
    /// <param name="device">The device defining output representation.</param>
    /// <param name="options">TeX engine run options.</param>
    ASPOSE_TEX_SHARED_API TeXJob(System::String path, System::SharedPtr<Presentation::Device> device, System::SharedPtr<TeXOptions> options);
    /// <summary>
    /// Creates a TeX job for running the engine in production mode to typeset a TeX document.
    /// The engine will prompt the file name as soon as it starts.
    /// Thus this run is supposed to be interactive.
    /// </summary>
    /// <param name="device">The device defining output representation.</param>
    /// <param name="options">TeX engine run options.</param>
    ASPOSE_TEX_SHARED_API TeXJob(System::SharedPtr<Presentation::Device> device, System::SharedPtr<TeXOptions> options);
    
    /// <summary>
    /// Runs TeX job.
    /// </summary>
    ASPOSE_TEX_SHARED_API void Run();
    /// <summary>
    /// Runs TeX engine in INITEX mode to create a format file (.fmt).
    /// </summary>
    /// <param name="path">The path to the main format source file.</param>
    /// <param name="options">TeX engine run options.</param>
    static ASPOSE_TEX_SHARED_API void CreateFormat(System::String path, System::SharedPtr<TeXOptions> options);
    
protected:

    System::SharedPtr<TeXEngine> get_Engine() const;
    
    #ifdef ASPOSE_GET_SHARED_MEMBERS
    ASPOSE_TEX_SHARED_API System::Object::shared_members_type GetSharedMembers() override;
    #endif
    
    
private:

    System::SharedPtr<TeXEngine> _engine;
    
};

} // namespace TeX
} // namespace Aspose


