﻿#pragma once
// Copyright (c) 2001-2021 Aspose Pty Ltd. All Rights Reserved.

#include <system/string.h>
#include <system/date_time.h>

#include "Aspose.TeX.Cpp/aspose_tex_api_defs.h"

namespace Aspose
{
namespace TeX
{
namespace Features
{
class MathRenderer;
} // namespace Features
enum class Interaction;
namespace IO
{
class IInputTerminal;
class IInputWorkingDirectory;
class IO;
class IOutputTerminal;
class IOutputWorkingDirectory;
} // namespace IO
namespace Logging
{
class TeXLog;
} // namespace Logging
namespace Presentation
{
class SaveOptions;
} // namespace Presentation
namespace Reading
{
class ReadingContext;
class TeXReader;
} // namespace Reading
namespace Tests
{
namespace Reading
{
class EquivTests;
} // namespace Reading
class TestBase;
} // namespace Tests
class TeXConfig;
class TeXEngine;
class TeXJob;
} // namespace TeX
} // namespace Aspose

namespace Aspose {

namespace TeX {

/// <summary>
/// TeX file processing options class.
/// </summary>
class ASPOSE_TEX_SHARED_CLASS TeXOptions : public System::Object
{
    typedef TeXOptions ThisType;
    typedef System::Object BaseType;
    
    typedef ::System::BaseTypesInfo<BaseType> ThisTypeBaseTypesInfo;
    ASPOSE_TEX_SHARED_RTTI_INFO_DECL();
    
    friend class Aspose::TeX::Features::MathRenderer;
    friend class Aspose::TeX::TeXEngine;
    friend class Aspose::TeX::Reading::ReadingContext;
    friend class Aspose::TeX::Reading::TeXReader;
    friend class Aspose::TeX::TeXJob;
    friend class Aspose::TeX::IO::IO;
    friend class Aspose::TeX::Tests::TestBase;
    friend class Aspose::TeX::Tests::Reading::EquivTests;
    
public:

    /// <summary>
    /// Gets/set the name of the job.
    /// </summary>
    ASPOSE_TEX_SHARED_API System::String get_JobName() const;
    /// <summary>
    /// Gets/set the name of the job.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_JobName(System::String value);
    /// <summary>
    /// Gets/sets the input terminal reader.
    /// </summary>
    ASPOSE_TEX_SHARED_API System::SharedPtr<IO::IInputTerminal> get_TerminalIn() const;
    /// <summary>
    /// Gets/sets the input terminal reader.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_TerminalIn(System::SharedPtr<IO::IInputTerminal> value);
    /// <summary>
    /// Gets/sets the output terminal writer.
    /// </summary>
    ASPOSE_TEX_SHARED_API System::SharedPtr<IO::IOutputTerminal> get_TerminalOut() const;
    /// <summary>
    /// Gets/sets the output terminal writer.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_TerminalOut(System::SharedPtr<IO::IOutputTerminal> value);
    /// <summary>
    /// Gets/sets input working directory.
    /// </summary>
    ASPOSE_TEX_SHARED_API System::SharedPtr<IO::IInputWorkingDirectory> get_InputWorkingDirectory() const;
    /// <summary>
    /// Gets/sets input working directory.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_InputWorkingDirectory(System::SharedPtr<IO::IInputWorkingDirectory> value);
    /// <summary>
    /// Gets/sets output working directory.
    /// </summary>
    ASPOSE_TEX_SHARED_API System::SharedPtr<IO::IOutputWorkingDirectory> get_OutputWorkingDirectory() const;
    /// <summary>
    /// Gets/sets output working directory.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_OutputWorkingDirectory(System::SharedPtr<IO::IOutputWorkingDirectory> value);
    /// <summary>
    /// Gets/sets the directory for the required input, e.g.,
    /// packages that are beyond Aspose.TeX's LaTeX support.
    /// </summary>
    ASPOSE_TEX_SHARED_API System::SharedPtr<IO::IInputWorkingDirectory> get_RequiredInputDirectory() const;
    /// <summary>
    /// Gets/sets the directory for the required input, e.g.,
    /// packages that are beyond Aspose.TeX's LaTeX support.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_RequiredInputDirectory(System::SharedPtr<IO::IInputWorkingDirectory> value);
    /// <summary>
    /// Gets/sets the interaction mode to run a TeX engine in.
    /// </summary>
    ASPOSE_TEX_SHARED_API Aspose::TeX::Interaction get_Interaction() const;
    /// <summary>
    /// Gets/sets the interaction mode to run a TeX engine in.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_Interaction(Aspose::TeX::Interaction value);
    /// <summary>
    /// Gets/sets the flag that instructs the engine whether to halt
    /// on missing package read attempt or ignore it.
    /// </summary>
    ASPOSE_TEX_SHARED_API bool get_IgnoreMissingPackages() const;
    /// <summary>
    /// Gets/sets the flag that instructs the engine whether to halt
    /// on missing package read attempt or ignore it.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_IgnoreMissingPackages(bool value);
    /// <summary>
    /// Gets/sets options used for rendering into destination format (XPS, PDF, image, etc.).
    /// Default value is the set of default options for rendering to XPS.
    /// </summary>
    ASPOSE_TEX_SHARED_API System::SharedPtr<Aspose::TeX::Presentation::SaveOptions> get_SaveOptions() const;
    /// <summary>
    /// Gets/sets options used for rendering into destination format (XPS, PDF, image, etc.).
    /// Default value is the set of default options for rendering to XPS.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_SaveOptions(System::SharedPtr<Aspose::TeX::Presentation::SaveOptions> value);
    /// <summary>
    /// Gets/sets a certain value for date/time primitives like \year, \month, \day and \time.
    /// </summary>
    ASPOSE_TEX_SHARED_API System::DateTime get_DateTime() const;
    /// <summary>
    /// Gets/sets a certain value for date/time primitives like \year, \month, \day and \time.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_DateTime(System::DateTime value);
    /// <summary>
    /// Gets/sets the flag indicating whether it is necessary to run the TeX job twice in case,
    /// for example, there are references in input TeX file(s). In general, this behavior is useful when
    /// the engine collects some data along the typesetting process and stores it in an auxilliary file,
    /// all at the first run. And at the second run, the engine somehow uses that data.
    /// </summary>
    ASPOSE_TEX_SHARED_API bool get_Repeat() const;
    /// <summary>
    /// Gets/sets the flag indicating whether it is necessary to run the TeX job twice in case,
    /// for example, there are references in input TeX file(s). In general, this behavior is useful when
    /// the engine collects some data along the typesetting process and stores it in an auxilliary file,
    /// all at the first run. And at the second run, the engine somehow uses that data.
    /// </summary>
    ASPOSE_TEX_SHARED_API void set_Repeat(bool value);
    
    /// <summary>
    /// Returns options for use in console application.
    /// </summary>
    /// <param name="config">A TeX config.</param>
    /// <returns>TeX options.</returns>
    static ASPOSE_TEX_SHARED_API System::SharedPtr<TeXOptions> ConsoleAppOptions(System::SharedPtr<TeXConfig> config);
    
protected:

    /// <summary>
    /// Gets the TeX engine configuration.
    /// </summary>
    System::SharedPtr<TeXConfig> get_Config() const;
    bool get_IniTeX() const;
    void set_IniTeX(bool value);
    System::SharedPtr<Logging::TeXLog> get_Log() const;
    void set_Log(System::SharedPtr<Logging::TeXLog> value);
    
    TeXOptions(System::SharedPtr<TeXConfig> config);
    
    MEMBER_FUNCTION_MAKE_OBJECT_DECLARATION(TeXOptions, CODEPORTING_ARGS(System::SharedPtr<TeXConfig> config));
    #ifdef ASPOSE_GET_SHARED_MEMBERS
    ASPOSE_TEX_SHARED_API System::Object::shared_members_type GetSharedMembers() override;
    #endif
    
    
private:

    System::SharedPtr<TeXConfig> _config;
    System::String _jobName;
    System::SharedPtr<IO::IInputTerminal> _terminalIn;
    System::SharedPtr<IO::IOutputTerminal> _terminalOut;
    System::SharedPtr<IO::IInputWorkingDirectory> _inputWorkingDirectory;
    System::SharedPtr<IO::IOutputWorkingDirectory> _outputWorkingDirectory;
    System::SharedPtr<IO::IInputWorkingDirectory> _requiredInputDirectory;
    Aspose::TeX::Interaction _interaction;
    bool _ignoreMissingPackages;
    System::SharedPtr<Aspose::TeX::Presentation::SaveOptions> _saveOptions;
    System::DateTime _dateTime;
    bool _repeat;
    bool _iniTeX;
    System::SharedPtr<Logging::TeXLog> _log;
    
};

} // namespace TeX
} // namespace Aspose


