/// @file drawing/known_color.h
#ifndef _aspose_system_known_color_h_
#define _aspose_system_known_color_h_

namespace System { namespace Drawing {
    /// Specifies the known system colors.
    enum class KnownColor
    {
        /// The color of the active window's border.
        ActiveBorder = 1,
        /// The color of the background of the active window's title bar.
        ActiveCaption = 2,
        /// The color of the text in the title bar of the active window.
        ActiveCaptionText = 3,
        /// The color of the application workspace.
        AppWorkspace = 4,
        /// The color of the face of a 3-Dimensional element.
        Control = 5,
        /// The color of the shadow of a 3-Dimensional element.
        ControlDark = 6,
        /// The color of the dark shadow of a 3-Dimensional element.
        ControlDarkDark = 7,        
        /// The color of the light of a 3-Dimensional element. 
        ControlLight = 8,
        /// The color of the highlight of a 3-Dimensional element.
        ControlLightLight = 9,
        /// The color of text in a 3-Dimensional element.
        ControlText = 10,
        /// The color of the desktop.
        Desktop = 11,
        /// The color of dimmed text.
        GrayText = 12,
        /// The seleceted items' background color.
        Highlight = 13,
        /// The color of the text of selected items.
        HighlightText = 14,
        /// The color of a hot-tracked item.
        HotTrack = 15,
        /// The color of the border of an inactive window.
        InactiveBorder = 16,
        /// The color of the background of an inactive window's title bar.
        InactiveCaption = 17,
        /// The color of the text in an inactive window's title bar.
        InactiveCaptionText = 18,
        /// The color of the ToolTip's background.
        Info = 19,
        /// The color of the text of a ToolTip.
        InfoText = 20,
        /// The color of the background of a menu.
        Menu = 21,
        /// The color of text in a menu.
        MenuText = 22,
        /// The color of the background of a scroll bar.
        ScrollBar = 23,
        /// The color of the background in the client area of a window.
        Window = 24,
        /// The color of a window frame.
        WindowFrame = 25,
        /// The color of the text in the client area of a window.
        WindowText = 26,
        /// The system-defined color whose ARGB value in hexadecimal notation is \#00FFFFFF.
        Transparent = 27,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFF0F8FF.
        AliceBlue = 28,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFAEBD7.
        AntiqueWhite = 29,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF00FFFF.
        Aqua = 30,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF7FFFD4.
        Aquamarine = 31,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFF0FFFF.
        Azure = 32,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFF5F5DC.
        Beige = 33,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFE4C4.
        Bisque = 34,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF000000.
        Black = 35,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFEBCD.
        BlanchedAlmond = 36,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF0000FF.
        Blue = 37,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF8A2BE2.
        BlueViolet = 38,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFA52A2A.
        Brown = 39,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFDEB887.
        BurlyWood = 40,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF5F9EA0.
        CadetBlue = 41,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF7FFF00.
        Chartreuse = 42,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFD2691E.
        Chocolate = 43,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFF7F50.
        Coral = 44,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF6495ED.
        CornflowerBlue = 45,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFF8DC.
        Cornsilk = 46,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFDC143C.
        Crimson = 47,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF00FFFF.
        Cyan = 48,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF00008B.
        DarkBlue = 49,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF008B8B.
        DarkCyan = 50,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFB8860B.
        DarkGoldenrod = 51,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFA9A9A9.
        DarkGray = 52,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF006400.
        DarkGreen = 53,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFBDB76B.
        DarkKhaki = 54,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF8B008B.
        DarkMagenta = 55,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF556B2F.
        DarkOliveGreen = 56,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFF8C00.
        DarkOrange = 57,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF9932CC.
        DarkOrchid = 58,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF8B0000.
        DarkRed = 59,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFE9967A.
        DarkSalmon = 60,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF8FBC8F.
        DarkSeaGreen = 61,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF483D8B.
        DarkSlateBlue = 62,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF2F4F4F.
        DarkSlateGray = 63,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF00CED1.
        DarkTurquoise = 64,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF9400D3.
        DarkViolet = 65,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFF1493.
        DeepPink = 66,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF00BFFF.
        DeepSkyBlue = 67,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF696969.
        DimGray = 68,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF1E90FF.
        DodgerBlue = 69,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFB22222.
        Firebrick = 70,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFFAF0.
        FloralWhite = 71,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF228B22.
        ForestGreen = 72,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFF00FF.
        Fuchsia = 73,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFDCDCDC.
        Gainsboro = 74,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFF8F8FF.
        GhostWhite = 75,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFD700.
        Gold = 76,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFDAA520.
        Goldenrod = 77,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF808080.
        Gray = 78,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF008000.
        Green = 79,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFADFF2F.
        GreenYellow = 80,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFF0FFF0.
        Honeydew = 81,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFF69B4.
        HotPink = 82,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFCD5C5C.
        IndianRed = 83,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF4B0082.
        Indigo = 84,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFFFF0.
        Ivory = 85,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFF0E68C.
        Khaki = 86,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFE6E6FA.
        Lavender = 87,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFF0F5.
        LavenderBlush = 88,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF7CFC00.
        LawnGreen = 89,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFFACD.
        LemonChiffon = 90,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFADD8E6.
        LightBlue = 91,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFF08080.
        LightCoral = 92,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFE0FFFF.
        LightCyan = 93,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFAFAD2.
        LightGoldenrodYellow = 94,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFD3D3D3.
        LightGray = 95,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF90EE90.
        LightGreen = 96,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFB6C1.
        LightPink = 97,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFA07A.
        LightSalmon = 98,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF20B2AA.
        LightSeaGreen = 99,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF87CEFA.
        LightSkyBlue = 100,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF778899.
        LightSlateGray = 101,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFB0C4DE.
        LightSteelBlue = 102,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFFFE0.
        LightYellow = 103,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF00FF00.
        Lime = 104,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF32CD32.
        LimeGreen = 105,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFAF0E6.
        Linen = 106,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFF00FF.
        Magenta = 107,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF800000.
        Maroon = 108,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF66CDAA.
        MediumAquamarine = 109,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF0000CD.
        MediumBlue = 110,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFBA55D3.
        MediumOrchid = 111,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF9370DB.
        MediumPurple = 112,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF3CB371.
        MediumSeaGreen = 113,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF7B68EE.
        MediumSlateBlue = 114,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF00FA9A.
        MediumSpringGreen = 115,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF48D1CC.
        MediumTurquoise = 116,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFC71585.
        MediumVioletRed = 117,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF191970.
        MidnightBlue = 118,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFF5FFFA.
        MintCream = 119,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFE4E1.
        MistyRose = 120,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFE4B5.
        Moccasin = 121,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFDEAD.
        NavajoWhite = 122,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF000080.
        Navy = 123,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFDF5E6.
        OldLace = 124,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF808000.
        Olive = 125,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF6B8E23.
        OliveDrab = 126,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFA500.
        Orange = 127,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFF4500.
        OrangeRed = 128,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFDA70D6.
        Orchid = 129,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFEEE8AA.
        PaleGoldenrod = 130,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF98FB98.
        PaleGreen = 131,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFAFEEEE.
        PaleTurquoise = 132,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFDB7093.
        PaleVioletRed = 133,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFEFD5.
        PapayaWhip = 134,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFDAB9.
        PeachPuff = 135,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFCD853F.
        Peru = 136,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFC0CB.
        Pink = 137,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFDDA0DD.
        Plum = 138,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFB0E0E6.
        PowderBlue = 139,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF800080.
        Purple = 140,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFF0000.
        Red = 141,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFBC8F8F.
        RosyBrown = 142,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF4169E1.
        RoyalBlue = 143,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF8B4513.
        SaddleBrown = 144,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFA8072.
        Salmon = 145,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFF4A460.
        SandyBrown = 146,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF2E8B57.
        SeaGreen = 147,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFF5EE.
        SeaShell = 148,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFA0522D.
        Sienna = 149,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFC0C0C0.
        Silver = 150,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF87CEEB.
        SkyBlue = 151,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF6A5ACD.
        SlateBlue = 152,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF708090.
        SlateGray = 153,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFFAFA.
        Snow = 154,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF00FF7F.
        SpringGreen = 155,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF4682B4.
        SteelBlue = 156,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFD2B48C.
        Tan = 157,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF008080.
        Teal = 158,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFD8BFD8.
        Thistle = 159,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFF6347.
        Tomato = 160,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF40E0D0.
        Turquoise = 161,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFEE82EE.
        Violet = 162,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFF5DEB3.
        Wheat = 163,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFFFFF.
        White = 164,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFF5F5F5.
        WhiteSmoke = 165,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FFFFFF00.
        Yellow = 166,
        
        /// The system-defined color whose ARGB value in hexadecimal notation is \#FF9ACD32.
        YellowGreen = 167,
        
        /// The face color of a 3-Dimensional element.
        ButtonFace = 168,
        
        /// The highlight color of a 3-Dimensional element.
        ButtonHighlight = 169,
        
        /// The color of the shadow of a 3-Dimensional element. 
        ButtonShadow = 170,
        
        /// The lightest color in the color gradient of the title bar of an active window.
        GradientActiveCaption = 171,
        
        /// The lightest color in the color gradient of the title bar of an inactive window.
        GradientInactiveCaption = 172,
        
        /// The color of the menu bar's background.
        MenuBar = 173,
        
        /// The color used to highlight menu items when the menu appears as a flat menu.
        MenuHighlight = 174
    };
}}

#endif
