/// @file system/make_const_ref.h
#pragma once

#include <system/smart_ptr.h>
#include <system/string.h>

namespace System {

/// Trait to make generic type "const reference" if it is String or a SmartPtr<> type
template <typename T>
struct MakeConstRef : std::conditional<IsSmartPtr<T>::value || std::is_same<System::String, T>::value, const T&, T> {};

/// Helper type for MakeConstRef modifier
template< class T >
using MakeConstRef_t = typename MakeConstRef<T>::type;

}